<?php

    namespace FederationServer\Classes;

    use FederationServer\Classes\Configuration\DatabaseConfiguration;
    use FederationServer\Classes\Configuration\RedisConfiguration;
    use FederationServer\Classes\Configuration\FileStorageConfiguration;

    class Configuration
    {
        private static ?\ConfigLib\Configuration $configuration = null;
        private static ?DatabaseConfiguration $databaseConfiguration = null;
        private static ?RedisConfiguration $redisConfiguration = null;
        private static ?FileStorageConfiguration $fileStorageConfiguration = null;

        /**
         * Initialize the configuration with default values.
         */
        public static function initialize(): void
        {
            self::$configuration = new \ConfigLib\Configuration('federation_server');

            self::$configuration->setDefault('database.host', '127.0.0.1');
            self::$configuration->setDefault('database.port', 3306);
            self::$configuration->setDefault('database.username', 'root');
            self::$configuration->setDefault('database.password', 'root');
            self::$configuration->setDefault('database.name', 'federation');
            self::$configuration->setDefault('database.charset', 'utf8mb4');
            self::$configuration->setDefault('database.collation', 'utf8mb4_unicode_ci');

            self::$configuration->setDefault('redis.enabled', true);
            self::$configuration->setDefault('redis.host', '127.0.1');
            self::$configuration->setDefault('redis.port', 6379);
            self::$configuration->setDefault('redis.password', null);
            self::$configuration->setDefault('redis.database', 0);

            self::$configuration->setDefault('file_storage.max_size', 52428800); // 50 MB
            self::$configuration->setDefault('file_storage.path', '/var/www/uploads');

            self::$configuration->save();

            self::$databaseConfiguration = new DatabaseConfiguration(self::$configuration->get('database'));
            self::$redisConfiguration = new RedisConfiguration(self::$configuration->get('redis'));
            self::$fileStorageConfiguration = new FileStorageConfiguration(self::$configuration->get('file_storage'));
        }

        /**
         * Get the configuration values.
         *
         * @return array
         */
        public static function getConfiguration(): array
        {
            if(self::$configuration === null)
            {
                self::initialize();
            }

            return self::$configuration->getConfiguration();
        }

        /**
         * Get the configuration library instance.
         *
         * @return \ConfigLib\Configuration
         */
        public static function getConfigurationLib(): \ConfigLib\Configuration
        {
            if(self::$configuration === null)
            {
                self::initialize();
            }

            return self::$configuration;
        }

        /**
         * Get the database, Redis, and file storage configurations.
         *
         * @return DatabaseConfiguration
         */
        public static function getDatabaseConfiguration(): DatabaseConfiguration
        {
            if(self::$databaseConfiguration === null)
            {
                self::initialize();
            }

            return self::$databaseConfiguration;
        }

        /**
         * Get the Redis configuration.
         *
         * @return RedisConfiguration
         */
        public static function getRedisConfiguration(): RedisConfiguration
        {
            if(self::$redisConfiguration === null)
            {
                self::initialize();
            }

            return self::$redisConfiguration;
        }

        /**
         * Get the file storage configuration.
         *
         * @return FileStorageConfiguration
         */
        public static function getFileStorageConfiguration(): FileStorageConfiguration
        {
            if(self::$fileStorageConfiguration === null)
            {
                self::initialize();
            }

            return self::$fileStorageConfiguration;
        }
    }

